==> log_analyzer_mcp-0.1.6.tar.gz: log_analyzer_mcp-0.1.6/docs/refactoring/log_analyzer_refactoring_v2.md <==
# Refactoring Plan for `log_analyzer_mcp` - v2

This document outlines the steps to refactor the `log_analyzer_mcp` repository, focusing on enhancing the log analysis capabilities and streamlining the project. This plan supersedes `log_analyzer_refactoring_v1.md`.

## Phase 1: Initial Setup and Dependency Resolution (Completed in v1)

This phase is considered complete as per `log_analyzer_refactoring_v1.md`. All initial setup, dependency resolution, internal import fixes, and missing file issues have been addressed.

- [x] **Project Structure Update:**
  - [x] Acknowledge new `src/log_analyzer_client` and `tests/log_analyzer_client` directories.
  - [x] Confirm `pyproject.toml` `[tool.hatch.build.targets.wheel].packages` includes both `src/log_analyzer_mcp` and `src/log_analyzer_client` to ensure they are packaged together. Example: `packages = ["src/log_analyzer_mcp", "src/log_analyzer_client"]`.
  - [x] Ensure `[tool.hatch.version].path` points to a shared root or primary module like `src/log_analyzer_mcp/__init__.py`.

## Phase 2: Core Log Analyzer Logic and Configuration

- [x] **Develop Core Analysis Module (in `src/log_analyzer_mcp/core` or similar):**
  - [x] Create a new module (e.g., `src/log_analyzer_mcp/core/analysis_engine.py`) to house the primary log parsing and filtering logic. This engine will be used by both the MCP server and the CLI client.
  - [x] Generalize log parsing to be highly flexible and configurable.
  - [x] Implement logic to search log file content based on filter criteria:
    - [x] **All records:** Include all matches based on defined patterns.
    - [x] **Time-based records:**
      - [x] Last N minutes.
      - [x] Last N hours.
      - [x] Last N days.
    - [x] **Positional records:**
      - [x] First N (oldest) records.
      - [x] Last N (newest) records.
  - [x] Implement log file filtering by:
    - [x] **Named logging scopes:** Allow users to define scopes in `.env` (e.g., `LOG_SCOPE_MODULE_A=logs/module_a/`, `LOG_SCOPE_SPECIFIC_FILE=logs/specific.log`) to focus search on specific directories or files.
  - [x] Implement flexible content filter match support, configurable via `.env`:
    - [x] **Log files directory/directories:** Define an array of directories to search within (e.g., `LOG_DIRECTORIES=["logs/", "another_log_dir/"]`). Default to searching all `*.log` files within the project root if not specified. **Ensure searches are always confined within the project directory.**
    - [x] **Specific search patterns per log level:** Allow an array of search patterns (strings or regex) for each log level (DEBUG, INFO, WARNING, ERROR) (e.g., `LOG_PATTERNS_ERROR=["Exception:.*", "Traceback (most recent call last):"]`).
    - [x] **Context lines:** Return N lines before and after a match (e.g., `LOG_CONTEXT_LINES_BEFORE=2`, `LOG_CONTEXT_LINES_AFTER=2`). Default to 2 lines before and 2 after if not specified.
  - [x] Ensure all configuration options read from `.env` can also be supplied via environment variables. This configuration loading should be part of the core module or a shared utility. (Handled by `ConfigLoader`)
- [x] **Refactor `log_analyzer.py` (in `src/log_analyzer_mcp`):**
  - [x] This file might become a wrapper or utility that leverages the new core analysis engine, or its relevant logic moved into the core engine. Its previous role as a direct script for test log analysis will be superseded. (pytest specific logic moved to `test_log_parser.py`)
  - [x] Identify any specific test log parsing logic from the old `log_analyzer.py` that is still relevant for `analyze_tests` MCP tool and integrate it into the core engine or a specialized part of `src/log_analyzer_mcp`. (Moved to `test_log_parser.py` and used by `analyze_tests`)
- [x] **Create `.env.template`:**
  - [x] Provide example configurations for all new features:
    - [x] Logging scopes.
    - [x] Log directories.
    - [x] Search patterns per log level.
    - [x] Context lines.
  (Created as `dotenv.template`)
- [x] **Refactor Type Hinting (Project-wide):**
  - [x] Throughout the project (both MCP and Client), especially in function signatures for MCP tools and CLI arguments, avoid using `Optional[Type]` or `Union[Type, None]`.
  - [x] Instead, provide default values for parameters to make them implicitly optional (e.g., `param: str = "default_value"` instead of `param: Optional[str] = None`). This is to ensure better compatibility with AI-driven IDEs and MCP client integrations.
  - [x] *Note: Refer to FastMCP documentation for best practices if further clarification is needed on MCP tool signature compatibility.*
- [x] **Shared Utilities (e.g., in `src/log_analyzer_mcp/common`):**
  - [x] Ensure `logger_setup.py` remains a shared utility.
  - [x] Consider if any other common logic (e.g., config loading, path resolution) should be placed in `common` for use by both `log_analyzer_mcp` and `log_analyzer_client`. (Created `ConfigLoader`, `utils.py`)

## Phase 3: MCP Server and CLI Implementation

- [x] **Update MCP Server (`src/log_analyzer_mcp/log_analyzer_mcp_server.py`):**
  - [x] **Remove `analyze_runtime_errors` tool and related logic. The new core analysis engine should cover general log searching.** (Mark as pending until the core engine is stable and functional). (Tool removed, function moved to `analyze_runtime_errors.py`)
  - [x] **Remove `parse_coverage` tool and its associated tests.** This functionality is confirmed to be no longer needed. (Removed)
  - [x] Implement new MCP server tools that utilize the core analysis engine from `src/log_analyzer_mcp/core/`:
    - [x] `search_log_all_records`: Searches for all matching records.
      - [x] Parameters: `scope: str = "default"`, `context_before: int = 2`, `context_after: int = 2` (and other relevant global configs like patterns, directories if not scope-defined).
    - [x] `search_log_time_based`: Searches records within a time window.
      - [x] Parameters: `minutes: int = 0`, `hours: int = 0`, `days: int = 0`, `scope: str = "default"`, `context_before: int = 2`, `context_after: int = 2`. (Ensure only one time unit can be effectively non-zero).
    - [x] `search_log_first_n_records`: Searches for the first N matching records.
      - [x] Parameters: `count: int`, `scope: str = "default"`, `context_before: int = 2`, `context_after: int = 2`.
    - [x] `search_log_last_n_records`: Searches for the last N matching records.
      - [x] Parameters: `count: int`, `scope: str = "default"`, `context_before: int = 2`, `context_after: int = 2`.
  - [x] Keep the existing `analyze_tests` tool, but refactor it to use the core analysis engine or specialized test log parsing logic if retained from the old `log_analyzer.py`. (Refactored to use `test_log_parser.analyze_pytest_log_content`)
  - [x] Ensure all MCP tool parameters adhere to the non-Optional/Union type hinting rule.
- [x] **Implement CLI (`src/log_analyzer_client/cli.py`):**
  - [x] Use `click` or `argparse` for the CLI interface.
  - [x] This CLI will also utilize the core analysis engine from `src/log_analyzer_mcp/core/`.
  - [x] Create script aliases for CLI invocation (e.g., `log-analyzer`) via `pyproject.toml` `[project.scripts]`.
  - [x] Provide sub-commands that mirror the MCP server tools with feature parity:
    - [x] `log-analyzer search all [--scope SCOPE] [--before LINES] [--after LINES]`
    - [x] `log-analyzer search time [--minutes M] [--hours H] [--days D] [--scope SCOPE] [--before LINES] [--after LINES]`
    - [x] `log-analyzer search first [--count N] [--scope SCOPE] [--before LINES] [--after LINES]`
    - [x] `log-analyzer search last [--count N] [--scope SCOPE] [--before LINES] [--after LINES]`
  - [x] Allow all configuration options (log directories, patterns, etc.) to be overridden via CLI arguments if not using scopes from `.env`.
  - [x] Ensure CLI parameters also adhere to the non-Optional/Union type hinting rule where applicable for internal consistency.

## Phase 4: Testing and Coverage

- [ ] **Update/Create Tests:**
  - [x] **In `tests/log_analyzer_mcp/`:**
    - [x] Write comprehensive tests for the core analysis engine (`src/log_analyzer_mcp/core/analysis_engine.py`).
    - [x] Write tests for the new `.env` configuration loading and environment variable overrides (if handled by a shared module in `src/log_analyzer_mcp/common`). (Covered by `AnalysisEngine` tests with `ConfigLoader`)
    - [x] Write tests for the new/updated MCP server tools in `log_analyzer_mcp_server.py`. (Tests written; core functionality confirmed via direct MCP calls, but automated tests like `test_quick_subset` remain `xfail` due to `server_session` fixture issues)
    - [x] **Remove tests related to `parse_coverage.py`.** (Done)
    - [x] **Adapt or remove tests for `analyze_runtime_errors.py` once the module is removed.** (Adapted, `test_analyze_runtime_errors.py` calls direct function)
    - [x] Update tests for `log_analyzer.py` if it's retained in any form, or remove them if its functionality is fully migrated. (Superseded by `test_log_parser.py` and `AnalysisEngine` tests)
  - [ ] **In `tests/log_analyzer_client/`:**
    - [ ] Write tests for the CLI functionality in `src/log_analyzer_client/cli.py`, mocking or using the core analysis engine.
- [ ] **Achieve and Maintain Test Coverage:**
  - [ ] Ensure overall project test coverage is >= 80%, covering both `log_analyzer_mcp` and `log_analyzer_client` modules. (Currently lower, needs improvement)
  - [ ] Specifically target >= 80% coverage for the core analysis engine and the new MCP/CLI interfaces. (`AnalysisEngine` coverage is good, MCP/CLI needs more)

## Phase 5: Documentation and Finalization

- [ ] **Update/Create Documentation:**
  - [ ] Update `README.md` for the standalone project:
    - [ ] Installation instructions (using `hatch`), noting that it installs both MCP server components and the CLI client.
    - [ ] Detailed usage instructions for the MCP server tools.
    - [ ] Detailed usage instructions for the CLI (`log-analyzer`), including all commands and options.
    - [ ] Clear explanation of how to configure logging scopes, directories, patterns, and context lines using `.env` files and environment variables (relevant for both MCP server and CLI).
    - [ ] Examples for `.env` configuration.
    - [ ] How to run tests (covering both `tests/log_analyzer_mcp` and `tests/log_analyzer_client`) and check coverage.
  - [ ] Update `docs/refactoring/README.md` to link to this v2 plan.
  - [x] Create or update other documents in `docs/` as needed (e.g., `docs/usage.md`, `docs/configuration.md`, `docs/architecture.md` briefly explaining the client/server structure).
- [x] **Linting and Formatting (Project-wide):**
  - [x] Run `black .` and `isort .` across `src/log_analyzer_mcp`, `src/log_analyzer_client`, `tests/log_analyzer_mcp`, `tests/log_analyzer_client`. (Done)
  - [ ] Run `pylint src tests` and address warnings/errors. (Skipped for now)
  - [ ] Run `mypy src tests` and address type errors, paying close attention to the new type hinting guidelines.
- [x] **Build and Distribution:**
  - [x] Verify `pyproject.toml` correctly defines `[project.scripts]` for the `log-analyzer` CLI. (Verified during CLI implementation)
  - [x] Test building a wheel: `hatch build`. Ensure both modules are included.
  - [x] If this package is intended for PyPI, ensure all metadata is correct.
- [ ] **Final Review:**
  - [ ] Review all changes and ensure the repository is clean, self-contained, and adheres to the new refactoring goals.
  - [ ] Ensure a consistent class hierarchy and code design is maintained, especially for shared components.
  - [x] Ensure all `.cursorrules` instructions are being followed.
  - [x] *Note on FastMCP: Consult the FastMCP documentation for any specific guidance on MCP server implementation details, especially regarding tool definitions and type handling, to ensure optimal compatibility. This can be fetched via the `mcp_FastMCP_Docs_fetch_fastmcp_documentation` tool if needed.* (Fetched)

## Deferred Tasks

- [x] **Remove `src/log_analyzer_mcp/analyze_runtime_errors.py` and its tests:** This will be done after the core analysis engine is complete and it's confirmed that no code from `analyze_runtime_errors.py` needs to be salvaged or migrated. (Function moved, module kept for now, tests adapted)

## Notes

- The primary goal is to create a highly flexible and configurable log analysis tool with a clear separation between the core logic (in `log_analyzer_mcp`), the MCP service interface (`log_analyzer_mcp`), and a command-line client (`log_analyzer_client`).
- Adherence to the specified type hinting style (no `Optional`/`Union` in favor of default values) is critical for broad compatibility.

==> log_analyzer_mcp-0.1.8.tar.gz: log_analyzer_mcp-0.1.8/docs/refactoring/log_analyzer_refactoring_v2.md <==
# Refactoring Plan for `log_analyzer_mcp` - v2

This document outlines the steps to refactor the `log_analyzer_mcp` repository, focusing on enhancing the log analysis capabilities and streamlining the project. This plan supersedes `log_analyzer_refactoring_v1.md`.

## Phase 1: Initial Setup and Dependency Resolution (Completed in v1)

This phase is considered complete as per `log_analyzer_refactoring_v1.md`. All initial setup, dependency resolution, internal import fixes, and missing file issues have been addressed.

- [x] **Project Structure Update:**
  - [x] Acknowledge new `src/log_analyzer_client` and `tests/log_analyzer_client` directories.
  - [x] Confirm `pyproject.toml` `[tool.hatch.build.targets.wheel].packages` includes both `src/log_analyzer_mcp` and `src/log_analyzer_client` to ensure they are packaged together. Example: `packages = ["src/log_analyzer_mcp", "src/log_analyzer_client"]`.
  - [x] Ensure `[tool.hatch.version].path` points to a shared root or primary module like `src/log_analyzer_mcp/__init__.py`.

## Phase 2: Core Log Analyzer Logic and Configuration

- [x] **Develop Core Analysis Module (in `src/log_analyzer_mcp/core` or similar):**
  - [x] Create a new module (e.g., `src/log_analyzer_mcp/core/analysis_engine.py`) to house the primary log parsing and filtering logic. This engine will be used by both the MCP server and the CLI client.
  - [x] Generalize log parsing to be highly flexible and configurable.
  - [x] Implement logic to search log file content based on filter criteria:
    - [x] **All records:** Include all matches based on defined patterns.
    - [x] **Time-based records:**
      - [x] Last N minutes.
      - [x] Last N hours.
      - [x] Last N days.
    - [x] **Positional records:**
      - [x] First N (oldest) records.
      - [x] Last N (newest) records.
  - [x] Implement log file filtering by:
    - [x] **Named logging scopes:** Allow users to define scopes in `.env` (e.g., `LOG_SCOPE_MODULE_A=logs/module_a/`, `LOG_SCOPE_SPECIFIC_FILE=logs/specific.log`) to focus search on specific directories or files.
  - [x] Implement flexible content filter match support, configurable via `.env`:
    - [x] **Log files directory/directories:** Define an array of directories to search within (e.g., `LOG_DIRECTORIES=["logs/", "another_log_dir/"]`). Default to searching all `*.log` files within the project root if not specified. **Ensure searches are always confined within the project directory.**
    - [x] **Specific search patterns per log level:** Allow an array of search patterns (strings or regex) for each log level (DEBUG, INFO, WARNING, ERROR) (e.g., `LOG_PATTERNS_ERROR=["Exception:.*", "Traceback (most recent call last):"]`).
    - [x] **Context lines:** Return N lines before and after a match (e.g., `LOG_CONTEXT_LINES_BEFORE=2`, `LOG_CONTEXT_LINES_AFTER=2`). Default to 2 lines before and 2 after if not specified.
  - [x] Ensure all configuration options read from `.env` can also be supplied via environment variables. This configuration loading should be part of the core module or a shared utility. (Handled by `ConfigLoader`)
- [x] **Refactor `log_analyzer.py` (in `src/log_analyzer_mcp`):**
  - [x] This file might become a wrapper or utility that leverages the new core analysis engine, or its relevant logic moved into the core engine. Its previous role as a direct script for test log analysis will be superseded. (pytest specific logic moved to `test_log_parser.py`)
  - [x] Identify any specific test log parsing logic from the old `log_analyzer.py` that is still relevant for `analyze_tests` MCP tool and integrate it into the core engine or a specialized part of `src/log_analyzer_mcp`. (Moved to `test_log_parser.py` and used by `analyze_tests`)
- [x] **Create `.env.template`:**
  - [x] Provide example configurations for all new features:
    - [x] Logging scopes.
    - [x] Log directories.
    - [x] Search patterns per log level.
    - [x] Context lines.
  (Created as `dotenv.template`)
- [x] **Refactor Type Hinting (Project-wide):**
  - [x] Throughout the project (both MCP and Client), especially in function signatures for MCP tools and CLI arguments, avoid using `Optional[Type]` or `Union[Type, None]`.
  - [x] Instead, provide default values for parameters to make them implicitly optional (e.g., `param: str = "default_value"` instead of `param: Optional[str] = None`). This is to ensure better compatibility with AI-driven IDEs and MCP client integrations.
  - [x] *Note: Refer to FastMCP documentation for best practices if further clarification is needed on MCP tool signature compatibility.*
- [x] **Shared Utilities (e.g., in `src/log_analyzer_mcp/common`):**
  - [x] Ensure `logger_setup.py` remains a shared utility.
  - [x] Consider if any other common logic (e.g., config loading, path resolution) should be placed in `common` for use by both `log_analyzer_mcp` and `log_analyzer_client`. (Created `ConfigLoader`, `utils.py`)

## Phase 3: MCP Server and CLI Implementation

- [x] **Update MCP Server (`src/log_analyzer_mcp/log_analyzer_mcp_server.py`):**
  - [x] **Remove `analyze_runtime_errors` tool and related logic. The new core analysis engine should cover general log searching.** (Mark as pending until the core engine is stable and functional). (Tool removed, function moved to `analyze_runtime_errors.py`)
  - [x] **Remove `parse_coverage` tool and its associated tests.** This functionality is confirmed to be no longer needed. (Removed)
  - [x] Implement new MCP server tools that utilize the core analysis engine from `src/log_analyzer_mcp/core/`:
    - [x] `search_log_all_records`: Searches for all matching records.
      - [x] Parameters: `scope: str = "default"`, `context_before: int = 2`, `context_after: int = 2` (and other relevant global configs like patterns, directories if not scope-defined).
    - [x] `search_log_time_based`: Searches records within a time window.
      - [x] Parameters: `minutes: int = 0`, `hours: int = 0`, `days: int = 0`, `scope: str = "default"`, `context_before: int = 2`, `context_after: int = 2`. (Ensure only one time unit can be effectively non-zero).
    - [x] `search_log_first_n_records`: Searches for the first N matching records.
      - [x] Parameters: `count: int`, `scope: str = "default"`, `context_before: int = 2`, `context_after: int = 2`.
    - [x] `search_log_last_n_records`: Searches for the last N matching records.
      - [x] Parameters: `count: int`, `scope: str = "default"`, `context_before: int = 2`, `context_after: int = 2`.
  - [x] Keep the existing `analyze_tests` tool, but refactor it to use the core analysis engine or specialized test log parsing logic if retained from the old `log_analyzer.py`. (Refactored to use `test_log_parser.analyze_pytest_log_content`)
  - [x] Ensure all MCP tool parameters adhere to the non-Optional/Union type hinting rule.
- [x] **Implement CLI (`src/log_analyzer_client/cli.py`):**
  - [x] Use `click` or `argparse` for the CLI interface.
  - [x] This CLI will also utilize the core analysis engine from `src/log_analyzer_mcp/core/`.
  - [x] Create script aliases for CLI invocation (e.g., `log-analyzer`) via `pyproject.toml` `[project.scripts]`.
  - [x] Provide sub-commands that mirror the MCP server tools with feature parity:
    - [x] `log-analyzer search all [--scope SCOPE] [--before LINES] [--after LINES]`
    - [x] `log-analyzer search time [--minutes M] [--hours H] [--days D] [--scope SCOPE] [--before LINES] [--after LINES]`
    - [x] `log-analyzer search first [--count N] [--scope SCOPE] [--before LINES] [--after LINES]`
    - [x] `log-analyzer search last [--count N] [--scope SCOPE] [--before LINES] [--after LINES]`
  - [x] Allow all configuration options (log directories, patterns, etc.) to be overridden via CLI arguments if not using scopes from `.env`.
  - [x] Ensure CLI parameters also adhere to the non-Optional/Union type hinting rule where applicable for internal consistency.

## Phase 4: Testing and Coverage

- [x] **Update/Create Tests:**
  - [x] **In `tests/log_analyzer_mcp/`:**
    - [x] Write comprehensive tests for the core analysis engine (`src/log_analyzer_mcp/core/analysis_engine.py`).
    - [x] Write tests for the new `.env` configuration loading and environment variable overrides (if handled by a shared module in `src/log_analyzer_mcp/common`). (Covered by `AnalysisEngine` tests with `ConfigLoader`)
    - [x] Write tests for the new/updated MCP server tools in `log_analyzer_mcp_server.py`. (Tests written; core functionality confirmed via direct MCP calls. `test_main_function_stdio_mode` successfully covers stdio startup via `main()`. `test_main_function_http_mode` is XFAIL. Other automated tests like `test_quick_subset` using the `server_session` fixture remain `xfail` due to fixture issues, though they currently `XPASS`.)
    - [x] **Remove tests related to `parse_coverage.py`.** (Done)
    - [x] **Adapt or remove tests for `analyze_runtime_errors.py` once the module is removed.** (Adapted, `test_analyze_runtime_errors.py` calls direct function)
    - [x] Update tests for `log_analyzer.py` if it's retained in any form, or remove them if its functionality is fully migrated. (Superseded by `test_log_parser.py` and `AnalysisEngine` tests)
  - [x] **In `tests/log_analyzer_client/`:**
    - [x] Write tests for the CLI functionality in `src/log_analyzer_client/cli.py`. (All 21 tests PASSING, achieving 100% coverage for `cli.py`)
- [ ] **Achieve and Maintain Test Coverage:**
  - [ ] Ensure overall project test coverage is >= 80%, covering both `log_analyzer_mcp` and `log_analyzer_client` modules. (Currently ~78% for `log_analyzer_mcp` and 100% for `log_analyzer_client`. `src/log_analyzer_client/cli.py` has 100% coverage. Key areas for improvement: `log_analyzer_mcp_server.py` (especially HTTP path if XFAIL resolved, and untested tools), and potentially `src/log_analyzer_mcp/test_log_parser.py`.)
  - [ ] Specifically target >= 80% coverage for the core analysis engine and the new MCP/CLI interfaces. (`AnalysisEngine` coverage is good; `src/log_analyzer_client/cli.py` is 100%. MCP server `main()` for HTTP mode (XFAIL) and other server tools need more test coverage.)

## Phase 5: Documentation and Finalization

- [ ] **Update/Create Documentation:**
  - [ ] Update `README.md` for the standalone project:
    - [ ] Installation instructions (using `hatch`), noting that it installs both MCP server components and the CLI client.
    - [ ] Detailed usage instructions for the MCP server tools.
    - [ ] Detailed usage instructions for the CLI (`log-analyzer`), including all commands and options.
    - [ ] Instructions on how to run the MCP server itself via its script entry point (e.g., `uvx log-analyzer-mcp` or `log-analyzer-mcp`), including the `--transport` option (`http` or `stdio`) and HTTP-specific options like `--host`, `--port`, and `--log-level`.
    - [ ] Clear explanation of how to configure logging scopes, directories, patterns, and context lines using `.env` files and environment variables (relevant for both MCP server and CLI).
    - [ ] Examples for `.env` configuration.
    - [ ] How to run tests (covering both `tests/log_analyzer_mcp` and `tests/log_analyzer_client`) and check coverage.
  - [ ] Update `docs/refactoring/README.md` to link to this v2 plan.
  - [x] Create or update other documents in `docs/` as needed (e.g., `docs/usage.md`, `docs/configuration.md`, `docs/architecture.md` briefly explaining the client/server structure).
- [x] **Linting and Formatting (Project-wide):**
  - [x] Run `black .` and `isort .` across `src/log_analyzer_mcp`, `src/log_analyzer_client`, `tests/log_analyzer_mcp`, `tests/log_analyzer_client`. (Done)
  - [ ] Run `pylint src tests` and address warnings/errors.
  - [ ] Run `mypy src tests` and address type errors, paying close attention to the new type hinting guidelines.
- [x] **Build and Distribution:**
  - [x] Verify `pyproject.toml` correctly defines `[project.scripts]` for the `log-analyzer` CLI. (Verified during CLI implementation)
  - [x] Test building a wheel: `hatch build`. Ensure both modules are included.
  - [x] If this package is intended for PyPI, ensure all metadata is correct.
- [ ] **Final Review:**
  - [ ] Review all changes and ensure the repository is clean, self-contained, and adheres to the new refactoring goals.
  - [ ] Ensure a consistent class hierarchy and code design is maintained, especially for shared components.
  - [x] Ensure all `.cursorrules` instructions are being followed.
  - [x] *Note on FastMCP: Consult the FastMCP documentation for any specific guidance on MCP server implementation details, especially regarding tool definitions and type handling, to ensure optimal compatibility. This can be fetched via the `mcp_FastMCP_Docs_fetch_fastmcp_documentation` tool if needed.* (Fetched)

## Deferred Tasks

- [x] **Remove `src/log_analyzer_mcp/analyze_runtime_errors.py` and its tests:** This will be done after the core analysis engine is complete and it's confirmed that no code from `analyze_runtime_errors.py` needs to be salvaged or migrated. (Function moved, module kept for now, tests adapted)

## Notes

- The primary goal is to create a highly flexible and configurable log analysis tool with a clear separation between the core logic (in `log_analyzer_mcp`), the MCP service interface (`log_analyzer_mcp`), and a command-line client (`log_analyzer_client`).
- Adherence to the specified type hinting style (no `Optional`/`Union` in favor of default values) is critical for broad compatibility.
